# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'analyse_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_AnalyseDialog(object):
    def setupUi(self, AnalyseDialog):
        AnalyseDialog.setObjectName("AnalyseDialog")
        AnalyseDialog.resize(791, 641)
        self.analyse = QtWidgets.QWidget(AnalyseDialog)
        self.analyse.setGeometry(QtCore.QRect(5, 130, 781, 506))
        font = QtGui.QFont()
        font.setFamily("Arial")
        self.analyse.setFont(font)
        self.analyse.setAutoFillBackground(True)
        self.analyse.setObjectName("analyse")
        self.btn_errorTime = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_errorTime.setGeometry(QtCore.QRect(145, 15, 111, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_errorTime.setFont(font)
        self.btn_errorTime.setObjectName("btn_errorTime")
        self.btn_errorOverview = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_errorOverview.setGeometry(QtCore.QRect(25, 15, 111, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_errorOverview.setFont(font)
        self.btn_errorOverview.setObjectName("btn_errorOverview")
        self.analyseBackground = QtWidgets.QLabel(AnalyseDialog)
        self.analyseBackground.setGeometry(QtCore.QRect(0, 0, 790, 126))
        self.analyseBackground.setText("")
        self.analyseBackground.setObjectName("analyseBackground")
        self.btn_errorAzAlt = QtWidgets.QPushButton(AnalyseDialog)
        self.btn_errorAzAlt.setGeometry(QtCore.QRect(265, 15, 111, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_errorAzAlt.setFont(font)
        self.btn_errorAzAlt.setObjectName("btn_errorAzAlt")
        self.groupBox = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox.setGeometry(QtCore.QRect(505, 10, 276, 106))
        self.groupBox.setObjectName("groupBox")
        self.checkOptimized = QtWidgets.QCheckBox(self.groupBox)
        self.checkOptimized.setGeometry(QtCore.QRect(15, 75, 256, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkOptimized.setFont(font)
        self.checkOptimized.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkOptimized.setChecked(False)
        self.checkOptimized.setObjectName("checkOptimized")
        self.winsorizeLimit = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.winsorizeLimit.setGeometry(QtCore.QRect(65, 45, 41, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(True)
        font.setWeight(75)
        self.winsorizeLimit.setFont(font)
        self.winsorizeLimit.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.winsorizeLimit.setDecimals(0)
        self.winsorizeLimit.setMinimum(1.0)
        self.winsorizeLimit.setMaximum(30.0)
        self.winsorizeLimit.setSingleStep(1.0)
        self.winsorizeLimit.setProperty("value", 5.0)
        self.winsorizeLimit.setObjectName("winsorizeLimit")
        self.checkWinsorize = QtWidgets.QCheckBox(self.groupBox)
        self.checkWinsorize.setGeometry(QtCore.QRect(15, 20, 111, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkWinsorize.setFont(font)
        self.checkWinsorize.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkWinsorize.setChecked(False)
        self.checkWinsorize.setObjectName("checkWinsorize")
        self.label_201 = QtWidgets.QLabel(self.groupBox)
        self.label_201.setGeometry(QtCore.QRect(30, 45, 31, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_201.setFont(font)
        self.label_201.setObjectName("label_201")
        self.label_202 = QtWidgets.QLabel(self.groupBox)
        self.label_202.setGeometry(QtCore.QRect(110, 45, 16, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_202.setFont(font)
        self.label_202.setObjectName("label_202")
        self.analyseBackground.raise_()
        self.btn_errorTime.raise_()
        self.btn_errorOverview.raise_()
        self.analyse.raise_()
        self.btn_errorAzAlt.raise_()
        self.groupBox.raise_()

        self.retranslateUi(AnalyseDialog)
        QtCore.QMetaObject.connectSlotsByName(AnalyseDialog)

    def retranslateUi(self, AnalyseDialog):
        _translate = QtCore.QCoreApplication.translate
        AnalyseDialog.setWindowTitle(_translate("AnalyseDialog", "Analyse"))
        self.btn_errorTime.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Shows the Ra Dec error of the aligment model from data.</p></body></html>"))
        self.btn_errorTime.setText(_translate("AnalyseDialog", "Error over time"))
        self.btn_errorOverview.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Shows an overview of the aligment model error from data.</p></body></html>"))
        self.btn_errorOverview.setText(_translate("AnalyseDialog", "Error overview"))
        self.analyseBackground.setProperty("color", _translate("AnalyseDialog", "blue"))
        self.btn_errorAzAlt.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Shows the Az Alt error of the aligment model from data.</p></body></html>"))
        self.btn_errorAzAlt.setText(_translate("AnalyseDialog", "Error over Az/Alt"))
        self.groupBox.setTitle(_translate("AnalyseDialog", "Optimize data for visibility"))
        self.checkOptimized.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Cuts outlayers by using winsorize method with the given limit.</p></body></html>"))
        self.checkOptimized.setText(_translate("AnalyseDialog", "Use data with pointing model correction"))
        self.winsorizeLimit.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p><br/></p></body></html>"))
        self.checkWinsorize.setToolTip(_translate("AnalyseDialog", "<html><head/><body><p>Cuts outlayers by using winsorize method with the given limit.</p></body></html>"))
        self.checkWinsorize.setText(_translate("AnalyseDialog", "Winsorize data"))
        self.label_201.setText(_translate("AnalyseDialog", "Limit"))
        self.label_202.setText(_translate("AnalyseDialog", "%"))

