# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'hemisphere_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_HemisphereDialog(object):
    def setupUi(self, HemisphereDialog):
        HemisphereDialog.setObjectName("HemisphereDialog")
        HemisphereDialog.resize(791, 641)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(HemisphereDialog.sizePolicy().hasHeightForWidth())
        HemisphereDialog.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Arial")
        HemisphereDialog.setFont(font)
        self.hemisphere = QtWidgets.QWidget(HemisphereDialog)
        self.hemisphere.setEnabled(True)
        self.hemisphere.setGeometry(QtCore.QRect(10, 130, 771, 501))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphere.sizePolicy().hasHeightForWidth())
        self.hemisphere.setSizePolicy(sizePolicy)
        self.hemisphere.setSizeIncrement(QtCore.QSize(0, 10))
        self.hemisphere.setAutoFillBackground(True)
        self.hemisphere.setStyleSheet("")
        self.hemisphere.setObjectName("hemisphere")
        self.btn_deletePoints = QtWidgets.QPushButton(HemisphereDialog)
        self.btn_deletePoints.setGeometry(QtCore.QRect(25, 15, 111, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_deletePoints.setFont(font)
        self.btn_deletePoints.setObjectName("btn_deletePoints")
        self.hemisphereBackground = QtWidgets.QLabel(HemisphereDialog)
        self.hemisphereBackground.setGeometry(QtCore.QRect(0, 0, 791, 126))
        self.hemisphereBackground.setText("")
        self.hemisphereBackground.setObjectName("hemisphereBackground")
        self.groupBox_2 = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox_2.setGeometry(QtCore.QRect(235, 10, 166, 106))
        self.groupBox_2.setObjectName("groupBox_2")
        self.checkEditNone = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkEditNone.setGeometry(QtCore.QRect(10, 20, 111, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEditNone.setFont(font)
        self.checkEditNone.setChecked(True)
        self.checkEditNone.setObjectName("checkEditNone")
        self.checkEditModelPoints = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkEditModelPoints.setGeometry(QtCore.QRect(10, 40, 146, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEditModelPoints.setFont(font)
        self.checkEditModelPoints.setObjectName("checkEditModelPoints")
        self.checkEditHorizonMask = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkEditHorizonMask.setGeometry(QtCore.QRect(10, 60, 141, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEditHorizonMask.setFont(font)
        self.checkEditHorizonMask.setObjectName("checkEditHorizonMask")
        self.checkPolarAlignment = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkPolarAlignment.setEnabled(True)
        self.checkPolarAlignment.setGeometry(QtCore.QRect(10, 80, 136, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkPolarAlignment.setFont(font)
        self.checkPolarAlignment.setObjectName("checkPolarAlignment")
        self.checkShowAlignmentStars = QtWidgets.QCheckBox(HemisphereDialog)
        self.checkShowAlignmentStars.setGeometry(QtCore.QRect(25, 50, 131, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowAlignmentStars.setFont(font)
        self.checkShowAlignmentStars.setObjectName("checkShowAlignmentStars")
        self.groupBox = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox.setGeometry(QtCore.QRect(420, 10, 356, 106))
        self.groupBox.setObjectName("groupBox")
        self.label_100 = QtWidgets.QLabel(self.groupBox)
        self.label_100.setGeometry(QtCore.QRect(10, 40, 66, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_100.setFont(font)
        self.label_100.setObjectName("label_100")
        self.le_numberPointsImaged = QtWidgets.QLineEdit(self.groupBox)
        self.le_numberPointsImaged.setEnabled(False)
        self.le_numberPointsImaged.setGeometry(QtCore.QRect(70, 60, 26, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_numberPointsImaged.setFont(font)
        self.le_numberPointsImaged.setMouseTracking(False)
        self.le_numberPointsImaged.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_numberPointsImaged.setAcceptDrops(False)
        self.le_numberPointsImaged.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_numberPointsImaged.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_numberPointsImaged.setReadOnly(True)
        self.le_numberPointsImaged.setObjectName("le_numberPointsImaged")
        self.label_77 = QtWidgets.QLabel(self.groupBox)
        self.label_77.setGeometry(QtCore.QRect(10, 60, 66, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_77.setFont(font)
        self.label_77.setObjectName("label_77")
        self.bar_numberPointsImaged = QtWidgets.QProgressBar(self.groupBox)
        self.bar_numberPointsImaged.setGeometry(QtCore.QRect(105, 60, 241, 16))
        self.bar_numberPointsImaged.setMaximum(1000)
        self.bar_numberPointsImaged.setProperty("value", 1)
        self.bar_numberPointsImaged.setTextVisible(False)
        self.bar_numberPointsImaged.setObjectName("bar_numberPointsImaged")
        self.bar_numberPointsSlewed = QtWidgets.QProgressBar(self.groupBox)
        self.bar_numberPointsSlewed.setGeometry(QtCore.QRect(105, 40, 241, 16))
        self.bar_numberPointsSlewed.setMinimum(0)
        self.bar_numberPointsSlewed.setMaximum(1000)
        self.bar_numberPointsSlewed.setProperty("value", 0)
        self.bar_numberPointsSlewed.setTextVisible(False)
        self.bar_numberPointsSlewed.setOrientation(QtCore.Qt.Horizontal)
        self.bar_numberPointsSlewed.setInvertedAppearance(False)
        self.bar_numberPointsSlewed.setTextDirection(QtWidgets.QProgressBar.BottomToTop)
        self.bar_numberPointsSlewed.setObjectName("bar_numberPointsSlewed")
        self.le_numberPointsSlewed = QtWidgets.QLineEdit(self.groupBox)
        self.le_numberPointsSlewed.setEnabled(False)
        self.le_numberPointsSlewed.setGeometry(QtCore.QRect(70, 40, 26, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_numberPointsSlewed.setFont(font)
        self.le_numberPointsSlewed.setMouseTracking(False)
        self.le_numberPointsSlewed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_numberPointsSlewed.setAcceptDrops(False)
        self.le_numberPointsSlewed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_numberPointsSlewed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_numberPointsSlewed.setReadOnly(True)
        self.le_numberPointsSlewed.setObjectName("le_numberPointsSlewed")
        self.le_numberPointsSolved = QtWidgets.QLineEdit(self.groupBox)
        self.le_numberPointsSolved.setEnabled(False)
        self.le_numberPointsSolved.setGeometry(QtCore.QRect(70, 80, 26, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_numberPointsSolved.setFont(font)
        self.le_numberPointsSolved.setMouseTracking(False)
        self.le_numberPointsSolved.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_numberPointsSolved.setAcceptDrops(False)
        self.le_numberPointsSolved.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_numberPointsSolved.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_numberPointsSolved.setReadOnly(True)
        self.le_numberPointsSolved.setObjectName("le_numberPointsSolved")
        self.bar_numberPointsSolved = QtWidgets.QProgressBar(self.groupBox)
        self.bar_numberPointsSolved.setGeometry(QtCore.QRect(105, 80, 241, 16))
        self.bar_numberPointsSolved.setMaximum(1000)
        self.bar_numberPointsSolved.setProperty("value", 1)
        self.bar_numberPointsSolved.setTextVisible(False)
        self.bar_numberPointsSolved.setObjectName("bar_numberPointsSolved")
        self.label_28 = QtWidgets.QLabel(self.groupBox)
        self.label_28.setGeometry(QtCore.QRect(10, 80, 66, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_28.setFont(font)
        self.label_28.setObjectName("label_28")
        self.le_numberPointsToProcess = QtWidgets.QLineEdit(self.groupBox)
        self.le_numberPointsToProcess.setEnabled(False)
        self.le_numberPointsToProcess.setGeometry(QtCore.QRect(125, 20, 26, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_numberPointsToProcess.setFont(font)
        self.le_numberPointsToProcess.setMouseTracking(False)
        self.le_numberPointsToProcess.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_numberPointsToProcess.setAcceptDrops(False)
        self.le_numberPointsToProcess.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_numberPointsToProcess.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_numberPointsToProcess.setReadOnly(True)
        self.le_numberPointsToProcess.setObjectName("le_numberPointsToProcess")
        self.label_29 = QtWidgets.QLabel(self.groupBox)
        self.label_29.setGeometry(QtCore.QRect(10, 20, 121, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_29.setFont(font)
        self.label_29.setObjectName("label_29")
        self.checkShowCelestial = QtWidgets.QCheckBox(HemisphereDialog)
        self.checkShowCelestial.setGeometry(QtCore.QRect(25, 70, 156, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowCelestial.setFont(font)
        self.checkShowCelestial.setObjectName("checkShowCelestial")
        self.checkShowMeridian = QtWidgets.QCheckBox(HemisphereDialog)
        self.checkShowMeridian.setGeometry(QtCore.QRect(25, 90, 156, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowMeridian.setFont(font)
        self.checkShowMeridian.setObjectName("checkShowMeridian")
        self.hemisphereStar = QtWidgets.QWidget(HemisphereDialog)
        self.hemisphereStar.setEnabled(True)
        self.hemisphereStar.setGeometry(QtCore.QRect(10, 130, 771, 501))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphereStar.sizePolicy().hasHeightForWidth())
        self.hemisphereStar.setSizePolicy(sizePolicy)
        self.hemisphereStar.setAutoFillBackground(False)
        self.hemisphereStar.setStyleSheet("")
        self.hemisphereStar.setObjectName("hemisphereStar")
        self.hemisphereMoving = QtWidgets.QWidget(HemisphereDialog)
        self.hemisphereMoving.setEnabled(True)
        self.hemisphereMoving.setGeometry(QtCore.QRect(10, 130, 771, 501))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphereMoving.sizePolicy().hasHeightForWidth())
        self.hemisphereMoving.setSizePolicy(sizePolicy)
        self.hemisphereMoving.setAutoFillBackground(False)
        self.hemisphereMoving.setStyleSheet("")
        self.hemisphereMoving.setObjectName("hemisphereMoving")
        self.hemisphereBackground.raise_()
        self.btn_deletePoints.raise_()
        self.groupBox_2.raise_()
        self.hemisphere.raise_()
        self.checkShowAlignmentStars.raise_()
        self.groupBox.raise_()
        self.checkShowCelestial.raise_()
        self.checkShowMeridian.raise_()
        self.hemisphereStar.raise_()
        self.hemisphereMoving.raise_()

        self.retranslateUi(HemisphereDialog)
        QtCore.QMetaObject.connectSlotsByName(HemisphereDialog)

    def retranslateUi(self, HemisphereDialog):
        _translate = QtCore.QCoreApplication.translate
        HemisphereDialog.setWindowTitle(_translate("HemisphereDialog", "Hemisphere"))
        self.btn_deletePoints.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p>Delete all points on coordinate window including Base / Model</p></body></html>"))
        self.btn_deletePoints.setText(_translate("HemisphereDialog", "Clear Points"))
        self.hemisphereBackground.setProperty("color", _translate("HemisphereDialog", "blue"))
        self.groupBox_2.setTitle(_translate("HemisphereDialog", "Operation Mode"))
        self.checkEditNone.setText(_translate("HemisphereDialog", "Normal mode"))
        self.checkEditModelPoints.setText(_translate("HemisphereDialog", "Edit Model Points"))
        self.checkEditHorizonMask.setText(_translate("HemisphereDialog", "Edit Horizon Mask"))
        self.checkPolarAlignment.setText(_translate("HemisphereDialog", "Select PolarAlign"))
        self.checkShowAlignmentStars.setText(_translate("HemisphereDialog", "Show align stars"))
        self.groupBox.setTitle(_translate("HemisphereDialog", "Mode Build Progress"))
        self.label_100.setText(_translate("HemisphereDialog", "Slewed:"))
        self.le_numberPointsImaged.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_numberPointsImaged.setText(_translate("HemisphereDialog", "0"))
        self.label_77.setText(_translate("HemisphereDialog", "Imaged:"))
        self.bar_numberPointsSlewed.setFormat(_translate("HemisphereDialog", "%p%"))
        self.le_numberPointsSlewed.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_numberPointsSlewed.setText(_translate("HemisphereDialog", "0"))
        self.le_numberPointsSolved.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_numberPointsSolved.setText(_translate("HemisphereDialog", "0"))
        self.label_28.setText(_translate("HemisphereDialog", "Solved:"))
        self.le_numberPointsToProcess.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_numberPointsToProcess.setText(_translate("HemisphereDialog", "0"))
        self.label_29.setText(_translate("HemisphereDialog", "Points to process:"))
        self.checkShowCelestial.setText(_translate("HemisphereDialog", "Show celestial paths"))
        self.checkShowMeridian.setText(_translate("HemisphereDialog", "Show meridian limits"))

